from typing import List

from dialog_api import sequence_and_updates_pb2

from dialog_bot_sdk.entities.Group import Group
from dialog_bot_sdk.entities.User import User
from dialog_bot_sdk.entities.message.Message import Message


class ReferencedEntities:
    def __init__(self, users: List[User], groups: List[Group], messages: List[Message]) -> None:
        self.users = users
        self.groups = groups
        self.messages = messages

    @classmethod
    def from_api(cls, entities: sequence_and_updates_pb2.RequestGetReferencedEntitites) -> 'ReferencedEntities':
        return cls([User.from_api(x) for x in entities.users], [Group.from_api(x) for x in entities.groups],
                   [Message.from_api(x) for x in entities.messages])



from typing import List

from dialog_api import users_pb2


class FullUser:
    def __init__(self, id: int, about: str, preferred_languages: List[str],
                 time_zone: str, custom_profile: str) -> None:
        self.id = id
        self.about = about
        self.preferred_languages = preferred_languages
        self.time_zone = time_zone
        self.custom_profile = custom_profile

    @classmethod
    def from_api(cls, full_user: users_pb2.FullUser) -> 'FullUser':
        return cls(full_user.id, full_user.about.value, [x for x in full_user.preferred_languages],
                   full_user.time_zone.value, full_user.custom_profile)

    def __dict__(self):
        return {"id": self.id, "about": self.about, "preferred_languages": self.preferred_languages,
                "time_zone": self.time_zone, "custom_profile": self.custom_profile}

    def __str__(self):
        return "FullUser({})".format(self.__dict__())



from dialog_bot_sdk.internal.bot import InternalBot
from dialog_bot_sdk.entity_manager import EntityManager


class ManagedService(object):
    """Base class for managed services.

    """
    def __init__(self, manager: EntityManager, internal: InternalBot) -> None:
        self.manager = manager
        self.internal = internal



from dialog_api import messaging_pb2

from dialog_bot_sdk.entities.Peer import Peer, PeerType
from dialog_bot_sdk.entities.UUID import UUID


class UpdateInteractiveMediaEvent:
    def __init__(self, mid: UUID, id_: str, value: str, peer: Peer) -> None:
        self.mid = mid
        self.id = id_
        self.value = value
        self.peer = peer

    @classmethod
    def from_api(cls, event: messaging_pb2.UpdateInteractiveMediaEvent) -> 'UpdateInteractiveMediaEvent':
        return cls(UUID.from_api(event.mid), event.id, event.value, Peer(event.uid, PeerType.PEERTYPE_PRIVATE))

    def __dict__(self):
        return {"mid": self.mid.__str__(), "id": self.id, "value": self.value, "peer": self.peer.__dict__()}

    def __str__(self):
        return "UpdateInteractiveMediaEvent({})".format(self.__dict__())



import logging
import grpc
import OpenSSL.crypto
import io
import copy

from grpc._channel import Channel

from .internal.bot import InternalBot
from .entity_manager import EntityManager
from .messaging import Messaging
from .updates import Updates
from .users import Users
from .groups import Groups
from .profile import Profile


DEFAULT_OPTIONS = {
    'grpc.keepalive_timeout_ms': 15000,
    'grpc.keepalive_time_ms': 30000,
    'grpc.keepalive_permit_without_calls': 1,
    'grpc.http2.max_pings_without_data': 0,
    'grpc.http2.min_time_between_pings_ms': 10000,
    'grpc.http2.min_ping_interval_without_data_ms': 5000,
    'grpc.min_reconnect_backoff_ms': 1000,
    'grpc.max_reconnect_backoff_ms': 30000
}


class DialogBot(object):
    """Main Dialog Bot class.
    """
    def __init__(self, channel: Channel, bot_token: str = None, verbose: bool = False, cert=None, private_key=None,
                 access_dir=None, options: dict = None):
        self.internal = InternalBot(channel, verbose=verbose, cert=cert, private_key=private_key, access_dir=access_dir,
                                    options=options)
        self.user_info = None
        if bot_token:
            self.user_info = self.internal.authorize(bot_token)
        else:
            self.user_info = self.internal.anonymous_authorize()
        self.manager = EntityManager(self.internal)
        self.groups = Groups(self.manager, self.internal)
        self.messaging = Messaging(self.manager, self.internal)
        self.updates = Updates(self.manager, self.internal)
        self.users = Users(self.manager, self.internal)
        self.profile = Profile(self.manager, self.internal)
        logging.info('Bot is ready.')

    @staticmethod
    def get_insecure_bot(endpoint: str, bot_token: str, verbose: bool = False, options: dict = None,
                         retry_options: dict = None) -> 'DialogBot':
        """Returns Dialog bot with established gRPC insecure channel.

        :param endpoint: bot's endpoint address
        :param bot_token: bot's token
        :param verbose: verbosity level of functions calling
        :param options: channel's options
        :param retry_options: dict of retries options (delay_factor, min_delay, max_delay, max_retries)
        :return: Dialog bot instance
        """
        options = DialogBot.__get_options(options)
        channel = grpc.insecure_channel(endpoint, options=options)
        return DialogBot(channel, bot_token, verbose=verbose, options=retry_options)

    @staticmethod
    def get_secure_bot(endpoint: str, credentials, bot_token: str, verbose: bool = False, options: dict = None,
                       retry_options: dict = None) -> 'DialogBot':
        """Returns Dialog bot with established gRPC insecure channel.

        :param endpoint: bot's endpoint address
        :param credentials: SSL credentials
        :param bot_token: bot's token
        :param verbose: verbosity level of functions calling
        :param options: channel's options
        :param retry_options: dict of retries options (delay_factor, min_delay, max_delay, max_retries)
        :return: Dialog bot instance
        """
        options = DialogBot.__get_options(options)
        channel = grpc.secure_channel(endpoint, credentials, options=options)
        return DialogBot(channel, bot_token, verbose=verbose, options=retry_options)

    @staticmethod
    def get_secure_bot_with_pfx_certificate(endpoint: str, pfx_certificate, pfx_password, verbose: bool = False,
                                            access_dir=None, options: dict = None,
                                            retry_options: dict = None) -> 'DialogBot':
        options = DialogBot.__get_options(options)
        pfx1 = open(pfx_certificate, 'rb').read()
        p12 = OpenSSL.crypto.load_pkcs12(pfx1, pfx_password)

        private_key = io.BytesIO()
        cert = io.BytesIO()

        private_key.write(OpenSSL.crypto.dump_privatekey(OpenSSL.crypto.FILETYPE_PEM, p12.get_privatekey()))
        cert.write(OpenSSL.crypto.dump_certificate(OpenSSL.crypto.FILETYPE_PEM, p12.get_certificate()))

        private_key.seek(0)
        cert.seek(0)

        channel = grpc.secure_channel(
            endpoint,
            grpc.ssl_channel_credentials(
                root_certificates=None,
                private_key=private_key.read(),
                certificate_chain=cert.read()
            ),
            options=options
        )

        private_key.seek(0)
        cert.seek(0)

        return DialogBot(channel, verbose=verbose, cert=cert.read(), private_key=private_key.read(),
                         access_dir=access_dir, options=retry_options)

    @staticmethod
    def __get_options(options: dict) -> list:
        options_dict = copy.deepcopy(DEFAULT_OPTIONS)
        if options:
            for key, value in options.items():
                options_dict[key] = value
        return list(options_dict.items())



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="dialog-bot-sdk",
    version="3.0.8",
    author="Dmitry Borodkin, Dialog LLC",
    author_email="d.borodkin@dlg.im",
    description="Python Bot SDK for Dialog Messenger",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dialogs/python-bot-sdk",
    packages=setuptools.find_packages(),
    license='Apache License 2.0',
    keywords='dialog messenger bot sdk',
    install_requires=[
        'dialog_api==1.53.0',
        'protobuf==3.10.0',
        'grpcio==1.24.1',
        'grpcio-tools==1.24.1',
        'google-api-python-client==1.7.11',
        'googleapis-common-protos==1.6.0',
        'requests==2.22.0',
        'pyopenssl==19.0.0',
        'Pillow==6.2.2',
    ],
    python_requires='>=3',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
)



from dialog_api import groups_pb2
from dialog_api.groups_pb2 import GROUPTYPE_GROUP, GROUPTYPE_UNKNOWN, GROUPTYPE_CHANNEL, GROUPTYPE_THREAD

from dialog_bot_sdk.entities.Avatar import Avatar
from dialog_bot_sdk.entities.Peer import PeerType, Peer


class GroupType:
    GROUPTYPE_UNKNOWN = GROUPTYPE_UNKNOWN
    GROUPTYPE_GROUP = GROUPTYPE_GROUP
    GROUPTYPE_CHANNEL = GROUPTYPE_CHANNEL
    GROUPTYPE_THREAD = GROUPTYPE_THREAD


class GroupData:
    def __init__(self, about: str, avatar: Avatar, name: str, title: str, type: int) -> None:
        self.about = about
        self.avatar = avatar
        self.name = name
        self.title = title
        self.type = type

    @classmethod
    def from_api(cls, data: groups_pb2.GroupData) -> 'GroupData':
        return cls(data.about.value, Avatar.from_api(data.avatar), data.shortname.value, data.title, data.group_type)

    def __dict__(self):
        return {"about": self.about, "avatar": self.avatar.__dict__(), "name": self.name, "title": self.title,
                "type": self.type}

    def __str__(self):
        return "GroupData({})".format(self.__dict__())


class Group:
    def __init__(self, id_: int, data: GroupData) -> None:
        self.data = data
        self.peer = Peer(id_, PeerType.PEERTYPE_GROUP)

    @classmethod
    def from_api(cls, group: groups_pb2.Group) -> 'Group':
        return cls(group.id, GroupData.from_api(group.data))

    def __dict__(self):
        return {"data": self.data.__dict__(), "peer": self.peer.__dict__()}

    def __str__(self):
        return "Group({})".format(self.__dict__())



import uuid

from dialog_api.definitions_pb2 import UUIDValue


class UUID:
    def __init__(self, msb: int, lsb: int) -> None:
        self.msb = msb
        self.lsb = lsb

    def to_api(self) -> UUIDValue:
        return UUIDValue(msb=self.msb, lsb=self.lsb)

    @classmethod
    def from_api(cls, uuid: UUIDValue) -> 'UUID':
        return cls(uuid.msb, uuid.lsb)

    def __dict__(self):
        return {"msb": self.msb, "lsb": self.lsb}

    def __str__(self):
        return "UUID({})".format(self.to_string())

    def to_string(self):
        return (self.digits(self.msb >> 32, 8) + "-" +
                self.digits(self.msb >> 16, 4) + "-" +
                self.digits(self.msb, 4) + "-" +
                self.digits(self.lsb >> 48, 4) + "-" +
                self.digits(self.lsb, 12))

    @staticmethod
    def digits(val: int, digits: int):
        hi = 1 << (digits * 4)
        return str(hex(hi | (val & (hi - 1))))[2:]
